/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Copyright (C) 2009 Canonical Services Ltd (www.canonical.com)
 *
 * Authors: Rodrigo Moya <rodrigo.moya@canonical.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include <libedataserver/e-source-list.h>
#include <libebook/e-book.h>
#include "contacts-picker-model.h"

typedef struct {
	ESourceList *source_list;
	GSList *books;
	GHashTable *selection;
} ContactsPickerModelPrivate;

static void
free_private_struct (gpointer data)
{
	ContactsPickerModelPrivate *priv = (ContactsPickerModelPrivate *) data;

	if (priv != NULL) {
		if (priv->selection != NULL) {
			g_hash_table_destroy (priv->selection);
			priv->selection = NULL;
		}

		if (priv->source_list != NULL) {
			g_object_unref (G_OBJECT (priv->source_list));
			priv->source_list = NULL;
		}

		while (priv->books != NULL) {
			EBook *book = (EBook *) priv->books->data;
			priv->books = g_slist_remove (priv->books, book);
			g_object_unref (G_OBJECT (book));
		}

		g_free (priv);
	}
}

static void
add_one_contact (GtkTreeModel *model, const gchar *name, const gchar *markup, const gchar *email, EContact *contact)
{
	GtkTreeIter new_row;
	EContactPhoto *photo = NULL;

	gtk_list_store_append (GTK_LIST_STORE (model), &new_row);
	gtk_list_store_set (GTK_LIST_STORE (model), &new_row,
			    CONTACTS_PICKER_MODEL_COLUMN_NAME, name,
				    CONTACTS_PICKER_MODEL_COLUMN_MARKUP, markup,
				    CONTACTS_PICKER_MODEL_COLUMN_EMAIL, email,
				    -1);

	if (contact != NULL) {
		photo = e_contact_get (contact, E_CONTACT_PHOTO);
		if (photo != NULL)
			photo = e_contact_get (contact, E_CONTACT_LOGO);
	}

	if (photo) {
		GdkPixbufLoader *loader = gdk_pixbuf_loader_new ();

		gdk_pixbuf_loader_write (loader, photo->data.inlined.data, photo->data.inlined.length, NULL);
		gdk_pixbuf_loader_close (loader, NULL);
		gtk_list_store_set (GTK_LIST_STORE (model), &new_row,
				    CONTACTS_PICKER_MODEL_COLUMN_PIXBUF, gdk_pixbuf_loader_get_pixbuf (loader),
				    -1);

		g_object_unref (loader);
	} else {
		GtkIconTheme *icon_theme = gtk_icon_theme_get_default ();

		gtk_list_store_set (GTK_LIST_STORE (model), &new_row,
				    CONTACTS_PICKER_MODEL_COLUMN_PIXBUF, gtk_icon_theme_load_icon (icon_theme, GTK_STOCK_ORIENTATION_PORTRAIT, 32, 0, NULL),
				    -1);
	}
}
static void
add_contacts (GtkTreeModel *model, GList *contacts)
{
	GList *l;
	ContactsPickerModelPrivate *priv;

	priv = g_object_get_data (G_OBJECT (model), "ContactsPickerModelPrivate");

	for (l = contacts; l != NULL; l = l->next) {
		EContact *contact = l->data;

		/* We add the selected items when searching, so ignore them here */
		if (g_hash_table_lookup (priv->selection, (gconstpointer) e_contact_get_const (contact, E_CONTACT_FULL_NAME)))
			continue;

		add_one_contact (model,
				 e_contact_get_const (contact, E_CONTACT_FULL_NAME),
				 e_contact_get_const (contact, E_CONTACT_FULL_NAME),
				 e_contact_get_const (contact, E_CONTACT_FIRST_EMAIL_ID),
				 contact);
	}
}

static void
book_opened_cb (EBook *book, EBookStatus status, gpointer user_data)
{
	GList *contacts;
	EBookQuery *query;
	GtkTreeModel *model = (GtkTreeModel *) user_data;

	if (status != E_BOOK_ERROR_OK) {
		g_warning ("Error opening addressbook: %d", status);
		return;
	}

	/* Get all contacts for this book */
	query = e_book_query_any_field_contains ("");
	e_book_get_contacts (book, query, &contacts, NULL);
	e_book_query_unref (query);

	add_contacts (model, contacts);
	
}

GtkTreeModel *
contacts_picker_model_new (void)
{
	ContactsPickerModelPrivate *priv;
	GtkTreeModel *model;
	GConfClient *conf_client;
	GSList *gl;
	GError *error = NULL;

	priv = g_new0 (ContactsPickerModelPrivate, 1);
	priv->selection = g_hash_table_new_full (g_str_hash, g_str_equal, (GDestroyNotify) g_free, (GDestroyNotify) g_free);

	/* Create the model */
	model = GTK_TREE_MODEL (gtk_list_store_new (4, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, GDK_TYPE_PIXBUF, NULL));
	g_object_set_data_full (G_OBJECT (model), "ContactsPickerModelPrivate", priv, (GDestroyNotify) free_private_struct);

	/* Get all addressbooks for this user */
	if (!e_book_get_addressbooks (&priv->source_list, &error)) {
		g_warning ("Could not get list of addressbooks: %s", error->message);
		g_error_free (error);

		return model;
	}

	for (gl = e_source_list_peek_groups (priv->source_list); gl != NULL; gl = gl->next) {
		GSList *sl;

		for (sl = e_source_group_peek_sources ((ESourceGroup *) gl->data); sl != NULL; sl = sl->next) {
			EBook *book;

			error = NULL;

			/* Open this addressbook asynchronously */
			book = e_book_new ((ESource *) sl->data, &error);
			if (book != NULL) {
				e_book_async_open (book, TRUE, (EBookCallback) book_opened_cb, model);
				priv->books = g_slist_append (priv->books, book);
			} else {
				g_warning ("Could not open addressbook %s: %s", e_source_get_uri (sl->data), error->message);
				g_error_free (error);
			}
		}
	}

	return model;
}

static void
foreach_hash_to_model_cb (gpointer key, gpointer value, gpointer data)
{
	gchar *markup;
	GtkTreeModel *model = GTK_TREE_MODEL (data);

	markup = g_strdup_printf ("<b><big>%s</big></b>", (const gchar *) key);
	add_one_contact (model, (const gchar *) key, (const gchar *) markup, (const gchar *) value, /* FIXME */ NULL);
}

void
contacts_picker_model_search (GtkTreeModel *model, const gchar *search_string)
{
	ContactsPickerModelPrivate *priv;
	GSList *l;

	priv = g_object_get_data (G_OBJECT (model), "ContactsPickerModelPrivate");

	/* Reset the model */
	gtk_list_store_clear (GTK_LIST_STORE (model));

	/* First add all selected contacts */
	g_hash_table_foreach (priv->selection, (GHFunc) foreach_hash_to_model_cb, model);

	/* Traverse all books */
	for (l = priv->books; l != NULL; l = l->next) {
		GList *contacts;
		EBookQuery *query;
		EBook *book = E_BOOK (l->data);

		if (!e_book_is_opened (book))
			continue;

		query = e_book_query_any_field_contains (search_string);
		e_book_get_contacts (book, query, &contacts, NULL);
		e_book_query_unref (query);

		add_contacts (model, contacts);
	}
}

void
contacts_picker_model_select_item (GtkTreeModel *model, GtkTreePath *path)
{
	GtkTreeIter iter;

	if (gtk_tree_model_get_iter (model, &iter, path)) {
		gchar *name, *email;
		ContactsPickerModelPrivate *priv;

		priv = g_object_get_data (G_OBJECT (model), "ContactsPickerModelPrivate");

		gtk_tree_model_get (model, &iter,
				    CONTACTS_PICKER_MODEL_COLUMN_NAME, &name,
				    CONTACTS_PICKER_MODEL_COLUMN_EMAIL, &email,
				    -1);

		/* if already selected, deselect it */
		if (g_hash_table_lookup (priv->selection, name)) {
			gtk_list_store_set (GTK_LIST_STORE (model), &iter,
					    CONTACTS_PICKER_MODEL_COLUMN_MARKUP, name,
					    -1);
			g_hash_table_remove (priv->selection, name);
		} else {
			gchar *markup;

			markup = g_strdup_printf ("<b><big>%s</big></b>", name);
			gtk_list_store_set (GTK_LIST_STORE (model), &iter,
					    CONTACTS_PICKER_MODEL_COLUMN_MARKUP, markup,
					    -1);
			g_free (markup);

			g_hash_table_insert (priv->selection, g_strdup (name), g_strdup (email));
		}
	}
}

GHashTable *
contacts_picker_model_get_selection (GtkTreeModel *model)
{
	ContactsPickerModelPrivate *priv;

	priv = g_object_get_data (G_OBJECT (model), "ContactsPickerModelPrivate");

	return priv->selection;
}
